<?php
/**
 * Operation abstract class
 *
 * @category    Inchoo
 * @package     Inchoo_CrossRelated
 * @author      Ivan Galambos <ivan.galambos@inchoo.net>
 */
abstract class Inchoo_CrossRelated_Model_Abstract extends Varien_Object
{
    /**
     * Error codes with arrays of corresponding row numbers.
     *
     * @var array
     */
    protected $_errors = array();

    /**
     * Has data process validation done?
     *
     * @var bool
     */
    protected $_dataValidated = false;

    /**
     * Array of numbers of validated rows as keys and boolean TRUE as values.
     *
     * @var array
     */
    protected $_validatedRows = array();

    /**
     * Number of rows processed by validation.
     *
     * @var int
     */
    protected $_processedRowsCount = 0;

    /**
     * Array of invalid rows numbers.
     *
     * @var array
     */
    protected $_invalidRows = array();

    /**
     * Number of invalid rows processed by validation.
     *
     * @var int
     */
    protected $_invalidRowsCount;

    /**
     * Error counter.
     *
     * @var int
     */
    protected $_errorsCount = 0;

    /**
     * Is all of data valid?
     *
     * @return bool
     */
    public function isDataValid($sourceFile)
    {
        $this->validateData($sourceFile);
        return 0 == $this->_errorsCount;
    }

    /**
     * Get Invalid rows count
     *
     * @return int
     */
    public function getInvalidRowsCount()
    {
        if (!$this->_invalidRowsCount) {
            $this->_invalidRowsCount = count($this->_invalidRows);
        }

        return $this->_invalidRowsCount;
    }

    /**
     * Get Processed rows count
     *
     * @return int
     */
    public function getProcessedRowsCount()
    {
        return $this->_processedRowsCount;
    }

    /**
     * Get Errors array
     *
     * @return array
     */
    public function getErrors()
    {
        if (!array($this->_invalidRows)) {
            $this->_invalidRows = array();
        }
        return $this->_invalidRows;
    }

    /**
     * Validate data.
     *
     * @return Inchoo_CrossRelated_Model_Abstract
     */
    public function validateData($sourceFile)
    {
        if (!$this->_dataValidated) {

            // initialize validation related attributes
            $this->_errors = array();
            $this->_invalidRows = array();

            $io = new Varien_Io_File();
            $io->streamOpen($sourceFile, 'r');

            while ( $row = $io->streamReadCsv() ) {

                $this->_processedRowsCount++;
                $checkThisRow = true;

                foreach ($row as $v) {
                    $v = trim($v);
                    if ( !ctype_digit($v) ) {
                        $checkThisRow = false;
                        $this->_errorsCount++;
                    }
                }

                if ( !$checkThisRow ) {
                    $this->_invalidRows[$this->_processedRowsCount] = $this->_processedRowsCount;
                }

            }

            $this->_dataValidated = true;

        }

        return $this;
    }

    /**
     * Validate data.
     *
     * @return Inchoo_CrossRelated_Model_Abstract
     */
    public function connectCsvDataToCrossRelated($row)
    {
        $productId = each($row);
        $productId = trim($productId[1]);

        $data = array();

        while ($value = each($row)) {
            $data[$value[1]] = array('position' => '0');
        }

        Mage::getResourceModel('catalog/product_link')->_initSaveProductLinksById((int)$productId, $data, Mage_Catalog_Model_Product_Link::LINK_TYPE_RELATED);

        return $this;
    }

}
