<?php

/**
 * Import model
 *
 * @category    Inchoo
 * @package     Inchoo_CrossRelated
 * @author      Ivan Galamboš <ivan.galambos@inchoo.net>
 */
class Inchoo_CrossRelated_Model_Import extends Inchoo_CrossRelated_Model_Abstract
{
    /**
     * Form field names (and IDs)
     */
    const FIELD_NAME_SOURCE_FILE = 'import_file';
    const FIELD_NAME_IMG_ARCHIVE_FILE = 'import_image_archive';

    /**
     * Validates source file and returns validation result.
     *
     * @param string $sourceFile Full path to source file
     * @return bool
     */
    public function validateSource($sourceFile)
    {
        $result = $this->isDataValid($sourceFile);

        return $result;
    }

    /**
     * Move uploaded file
     *
     * @throws Mage_Core_Exception
     * @return string Source file path
     */
    public function uploadSource()
    {
        $uploader  = Mage::getModel('core/file_uploader', self::FIELD_NAME_SOURCE_FILE);
        $uploader->skipDbProcessing(true);
        $result    = $uploader->save(self::getWorkingDir());
        $extension = pathinfo($result['file'], PATHINFO_EXTENSION);

        $uploadedFile = $result['path'] . $result['file'];

        if (!$extension) {
            unlink($uploadedFile);
            Mage::throwException(Mage::helper('crossrelated')->__('Uploaded file has no extension'));
        }

        $sourceFile = self::getWorkingDir() . 'crossrelated';
        $sourceFile .= '.' . $extension;

        if(strtolower($uploadedFile) != strtolower($sourceFile)) {

            if (file_exists($sourceFile)) {
                unlink($sourceFile);
            }

            if (!@rename($uploadedFile, $sourceFile)) {
                Mage::throwException(Mage::helper('crossrelated')->__('Source file moving failed'));
            }
        }

        return $sourceFile;
    }

    /**
     * Import/Export working directory (source files, result files, lock files etc.).
     *
     * @return string
     */
    public static function getWorkingDir()
    {
        return Mage::getBaseDir('var') . DS . 'crossrelated' . DS;
    }

    /**
     * Import & unlink checked crossrelated csv file.
     *
     * @throws Mage_Core_Exception
     * @return void
     */
    public function importSource()
    {
        $sourceFile = self::getWorkingDir() . 'crossrelated';
        $sourceFile .= '.' . 'csv';

        if (file_exists($sourceFile)) {

            $io = new Varien_Io_File();
            $io->streamOpen($sourceFile, 'r');

            while ( $row = $io->streamReadCsv() ) {

                $checkThisRow = true;

                foreach ($row as $v) {

                    $v = trim($v);
                    if ( !ctype_digit($v) ) {
                        $checkThisRow = false;
                    }

                }

                if ( $checkThisRow ) {
                    $this->connectCsvDataToCrossRelated($row);
                }
            }

            unlink($sourceFile);

        } else {
            Mage::throwException(Mage::helper('crossrelated')->__('Source file does not exist'));
        }

    }

}
