<?php

/**
 * Catalog product link resource model
 *
 * @category    Inchoo
 * @package     Inchoo_CrossRelated
 * @author      Ivan Galamboš <ivan.galambos@inchoo.net>
 */
class Inchoo_CrossRelated_Model_Resource_Product_Link extends Mage_Catalog_Model_Resource_Product_Link
{
    /**
     * Initialize Save Product Links process by ID#
     *
     * @param int $productId
     * @param array $data
     * @param int $typeId
     * @return Inchoo_CrossRelated_Model_Resource_Product_Link
     */
    public function _initSaveProductLinksById($productId, $data, $typeId)
    {
        if ( !(is_array($data) && count($data) && $productId) ) {
            return $this;
        }

        $data[$productId] = array('position' => '0');

        foreach ($data as $productId => $v) {
            /*
             * Note that we are not checking is $productId is valid or not, same as is valid $product->getRelatedReadonly()
             * */
            $productsCopy = $data;
            unset ($productsCopy[$productId]);
            $this->saveProductLinksById($productId, $productsCopy, $typeId);
        }

        return $this;
    }

    /**
     * Save Product Links process by ID#
     *
     * @param int $productId
     * @param array $data
     * @param int $typeId
     * @return Inchoo_CrossRelated_Model_Resource_Product_Link
     */
    public function saveProductLinksById($productId, $data, $typeId)
    {
        if (!is_array($data)) {
            $data = array();
        }

        $attributes = $this->getAttributesByType($typeId);
        $adapter    = $this->_getWriteAdapter();

        $bind   = array(
            ':product_id'    => (int)$productId,
            ':link_type_id'  => (int)$typeId
        );
        $select = $adapter->select()
            ->from($this->getMainTable(), array('linked_product_id', 'link_id'))
            ->where('product_id = :product_id')
            ->where('link_type_id = :link_type_id');

        $links   = $adapter->fetchPairs($select, $bind);

        $config = false;
        $configOption = Mage::helper('crossrelated')->getConfigEnabled();
        if ($configOption) {
            $config = Mage::helper('crossrelated')->getConfigRemove();
        }
        
        $deleteIds = array();
        $deleteProductIds = array();
        foreach($links as $linkedProductId => $linkId) {
            if (!isset($data[$linkedProductId])) {
                $deleteIds[] = (int)$linkId;
                if ($config) {
                    $deleteProductIds[] = (int)$linkedProductId;
                }
            }
        }

        if (!empty($deleteIds)) {
            $adapter->delete($this->getMainTable(), array(
                'link_id IN (?)' => $deleteIds,
            ));
        }

        if (!empty($deleteProductIds)) {
            $adapter->delete($this->getMainTable(), array(
                'product_id IN (?)' => $deleteProductIds,
            ));
        }

        foreach ($data as $linkedProductId => $linkInfo) {
            $linkId = null;
            if (isset($links[$linkedProductId])) {
                $linkId = $links[$linkedProductId];
                unset($links[$linkedProductId]);
            } else {
                $bind = array(
                    'product_id'        => $productId,
                    'linked_product_id' => $linkedProductId,
                    'link_type_id'      => $typeId
                );
                $adapter->insert($this->getMainTable(), $bind);
                $linkId = $adapter->lastInsertId($this->getMainTable());
            }

            foreach ($attributes as $attributeInfo) {
                $attributeTable = $this->getAttributeTypeTable($attributeInfo['type']);
                if ($attributeTable) {
                    if (isset($linkInfo[$attributeInfo['code']])) {
                        $value = $this->_prepareAttributeValue($attributeInfo['type'],
                            $linkInfo[$attributeInfo['code']]);
                        $bind = array(
                            'product_link_attribute_id' => $attributeInfo['id'],
                            'link_id'                   => $linkId,
                            'value'                     => $value
                        );
                        $adapter->insertOnDuplicate($attributeTable, $bind, array('value'));
                    } else {
                        $adapter->delete($attributeTable, array(
                            'link_id = ?'                   => $linkId,
                            'product_link_attribute_id = ?' => $attributeInfo['id']
                        ));
                    }
                }
            }
        }

        return $this;
    }
}
