<?php

require_once 'Mage/Adminhtml/controllers/Catalog/ProductController.php';

/**
 * Catalog product controller
 *
 * @category    Inchoo
 * @package     Inchoo_CrossRelated
 * @author      Ivan Galamboš <ivan.galambos@gmail.com>
 */
class Inchoo_CrossRelated_Adminhtml_Catalog_ProductController extends Mage_Adminhtml_Catalog_ProductController
{
    /**
     * Initialize layout.
     *
     * @return Inchoo_CrossRelated_Adminhtml_Catalog_ProductController
     */
    protected function _initAction()
    {
        $this->_title($this->__('Inchoo CrossRelated Products'))
            ->loadLayout()
        ;

        return $this;
    }

    /**
     * Import CrossRelated CSV data
     */
    public function csvimportAction()
    {
        $maxUploadSize = Mage::helper('crossrelated')->getMaxUploadSize();
        $this->_getSession()->addNotice(
            $this->__('Total size of uploadable files must not exceed %s', $maxUploadSize)
        );
        $this->_initAction()
            ->_title($this->__('CrossRelate CSV Import'))
            ->_addBreadcrumb($this->__('CrossRelate CSV Import'), $this->__('CrossRelate CSV Import'));
        $this->_setActiveMenu('catalog/products');
        $this->renderLayout();
    }

    /**
     * Start validation process action.
     *
     * @return void
     */
    public function validatecsvimportAction()
    {
        $data = $this->getRequest()->getPost();

        if ($data) {

            $this->loadLayout(false);
            /** @var $resultBlock Inchoo_CrossRelated_Block_Adminhtml_Import_Frame_Result */
            $resultBlock = $this->getLayout()->getBlock('import.frame.result.crossrelated');

            try {
                /** @var $import Inchoo_CrossRelated_Model_Import */
                $import = Mage::getModel('crossrelated/import');

                $validationResult = $import->validateSource($import->setData($data)->uploadSource());

                if (!$import->getProcessedRowsCount()) {
                    $resultBlock->addError($this->__('File does not contain data. Please upload another one'));
                } else {

                    if (!$validationResult) {

                        if ($import->getProcessedRowsCount() == $import->getInvalidRowsCount()) {
                            $resultBlock->addNotice(
                                $this->__('File is totally invalid. Please fix errors and re-upload file')
                            );
                        } else {
                                $resultBlock->addNotice(
                                    $this->__('Please fix errors and re-upload file or simply press "Import" button to skip rows with errors'),
                                    true
                                );
                        }
                        $error = $this->__('Invalid data (non-digits) in rows:') . ' ' . implode(', ', $import->getErrors());
                        $resultBlock->addError($error);

                    } else {
                            $resultBlock->addSuccess(
                                $this->__('File is valid! To start import process press "Import" button'), true
                            );
                    }
                    $resultBlock->addNotice($this->__('Checked rows: %d, invalid rows: %d', $import->getProcessedRowsCount(), $import->getInvalidRowsCount()));
                }

            } catch (Exception $e) {
                $resultBlock->addNotice($this->__('Please fix errors and re-upload file'))
                    ->addError($e->getMessage());
            }
            $this->renderLayout();

        } elseif ($this->getRequest()->isPost() && empty($_FILES)) {

            $this->loadLayout(false);
            $resultBlock = $this->getLayout()->getBlock('import.frame.result.crossrelated');
            $resultBlock->addError($this->__('File was not uploaded'));
            $this->renderLayout();

        } else {

            $this->_getSession()->addError($this->__('Data is invalid or file is not uploaded'));
            $this->_redirect('*/*/csvimport');
            return;

        }
        
        return;
    }

    /**
     * Start import process action.
     *
     * @return void
     */
    public function startcrossrelatedimportAction()
    {
        try {

            $this->loadLayout(false);

            /** @var $resultBlock Inchoo_CrossRelated_Block_Adminhtml_Import_Frame_Result */
            $resultBlock = $this->getLayout()->getBlock('import.frame.result.crossrelated');

            /** @var $import Inchoo_CrossRelated_Model_Import */
            $import = Mage::getModel('crossrelated/import');
            $import->importSource();

            $resultBlock->addAction('show', 'import_validation_container')
                ->addAction('innerHTML', 'import_validation_container_header', $this->__('Status'));

        } catch (Exception $e) {

            $resultBlock->addError($e->getMessage());
            $this->renderLayout();
            return;

        }

        $resultBlock->addAction('hide', array('edit_form', 'upload_button', 'messages'));
        $resultBlock->addSuccessRefresh($this->__('Import successfully done.'), true);

        $this->renderLayout();

        return;
    }

}
